@@ -32,6 +32,7 @@ from calculate.lib.utils.common import (getKernelUid,getTupleVersion,
     getValueFromCmdLine)
 from calculate.lib.utils.device import getUdevDeviceInfo
 from itertools import *
+from contextlib import contextmanager
 from calculate.install.distr import DistributiveError
 
 class VariableOsInstallKernelScheduler(Variable):
@@ -56,10 +57,12 @@ class VariableOsInstallKernelScheduler(Variable):
                                where='os_install_disk_mount',
                                eq='/',limit=1)
         if root_dev:
-            dev_ssd = self.Select('os_device_ssd_set',
+            dev_ssd, dev_virtual = self.Select(['os_device_ssd_set',
+                                                'os_device_virtual_set'],
                                    where='os_device_dev',
                                    eq=root_dev,limit=1)
-            if dev_ssd == 'on' and self.check_scheduler("noop"):
+            if ((dev_ssd == 'on' or dev_virtual == 'on')
+                and self.check_scheduler("noop")):
                 return "noop"
         return self.Get('os_install_kernel_schedule_default')
 
@@ -96,25 +99,32 @@ class VariableOsInstallKernelScheduler(Variable):
             return _("I/O scheduler unavailable for Flash install")
 
 
-class VariableOsInstallKernelScheduleDefault(Variable):
-    """
-    IO планировщик по умолчанию
-    """
-    type = "choice"
-
-    def get(self):
+class KernelConfigHelper:
+    @contextmanager
+    def kernel_config_lines(self):
         image = self.Get('cl_image')
         if image:
             with image as distr:
                 try:
                     distrPath = image.getDirectory()
                     kernelConfig = path.join(distrPath,"usr/src/linux/.config")
-                    for line in readLinesFile(kernelConfig):
-                        if "CONFIG_DEFAULT_IOSCHED=" in line:
-                            key, op, value = line.partition("=")
-                            return value.strip('"')
+                    yield readLinesFile(kernelConfig)
                 except DistributiveError as e:
                     pass
+
+
+class VariableOsInstallKernelScheduleDefault(KernelConfigHelper, Variable):
+    """
+    IO планировщик по умолчанию
+    """
+    type = "choice"
+
+    def get(self):
+        with self.kernel_config_lines() as kcl:
+            for line in kcl:
+                if "CONFIG_DEFAULT_IOSCHED=" in line:
+                    key, op, value = line.partition("=")
+                    return value.strip('"')
         return "cfq"
 
     def choice(self):
@@ -123,7 +133,7 @@ class VariableOsInstallKernelScheduleDefault(Variable):
                                      eq='on')
 
 
-class VariableOsInstallKernelScheduleData(ReadonlyTableVariable):
+class VariableOsInstallKernelScheduleData(KernelConfigHelper,ReadonlyTableVariable):
     """
     Information about kernel schedule
     """
@@ -131,22 +141,15 @@ class VariableOsInstallKernelScheduleData(ReadonlyTableVariable):
               'os_install_kernel_schedule_set']
 
     def get(self,hr=False):
-        image = self.Get('cl_image')
         installed = ['cfq']
         schedulers = {'CONFIG_IOSCHED_BFQ=y':'bfq',
                      'CONFIG_IOSCHED_NOOP=y':'noop',
                      'CONFIG_IOSCHED_CFQ=y':'cfq',
                      'CONFIG_IOSCHED_DEADLINE=y':'deadline'}
-        if image:
-            with image as distr:
-                try:
-                    distrPath = image.getDirectory()
-                    kernelConfig = path.join(distrPath,"usr/src/linux/.config")
-                    installed =  map(schedulers.get,
+        with self.kernel_config_lines() as kcl:
+            installed = map(schedulers.get,
                         filter(lambda x:x in schedulers,
-                        readLinesFile(kernelConfig)))
-                except DistributiveError as e:
-                    pass
+                        kcl)) or ['cfq']
         return [[x,"on" if x in installed else "off"]
                 for x in sorted(schedulers.values())]
 
@@ -168,24 +171,28 @@ class VariableOsInstallKernelScheduleSet(FieldValue,ReadonlyVariable):
     source_variable = "os_install_kernel_schedule_data"
     column = 1
 
-class VariableOsInstallKernelBfqSet(ReadonlyVariable):
+class VariableOsInstallKernelTuxoniceSet(KernelConfigHelper,ReadonlyVariable):
     """
     Available BFQ in kernel
     """
     type = "bool"
 
     def get(self):
-        image = self.Get('cl_image')
-        if image:
-            with image as distr:
-                try:
-                    distrPath = image.getDirectory()
-                    kernelConfig = path.join(distrPath,"usr/src/linux/.config")
-                    if filter(lambda x:"CONFIG_IOSCHED_BFQ=y" in x,
-                        readLinesFile(kernelConfig)):
-                        return "on"
-                except DistributiveError as e:
-                    return "off"
+        with self.kernel_config_lines() as kcl:
+            if any("CONFIG_TOI_CORE=y" in x for x in kcl):
+                return "on"
+        return "off"
+
+class VariableOsInstallKernelBfqSet(KernelConfigHelper,ReadonlyVariable):
+    """
+    Available BFQ in kernel
+    """
+    type = "bool"
+
+    def get(self):
+        with self.kernel_config_lines() as kcl:
+            if any("CONFIG_IOSCHED_BFQ=y" in x for x in kcl):
+                return "on"
         return "off"
 
 class VariableOsInstallKernelAttr(Variable):
@@ -216,7 +223,7 @@ class VariableOsInstallKernelResume(ReadonlyVariable):
         for dev, install in zip(self.Get('os_install_disk_use'),
                                 self.Get('os_install_disk_mount')):
             if install == "swap":
-                if self.Get('os_install_linux_system') == 'desktop':
+                if self.Get('os_install_kernel_tuxonice_set') == 'on':
                     return \
                         "tuxonice tuxonice_resume=%s real_resume=%s"%(dev,dev)
                 else:
